@@ -1,8 +1,7 @@
 /*
  * Copyright (c) 2003-2006 Erez Zadok
  * Copyright (c) 2003-2006 Charles P. Wright
- * Copyright (c) 2005-2006 Josef 'Jeff' Sipek
- * Copyright (c) 2005-2006 Junjiro Okajima
+ * Copyright (c) 2005-2006 Josef Sipek
  * Copyright (c) 2005      Arun M. Krishnakumar
  * Copyright (c) 2005-2006 David P. Quigley
  * Copyright (c) 2003-2004 Mohammad Nayyer Zubair
@@ -17,7 +16,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: dirfops.c,v 1.25 2006/08/05 01:28:46 jro Exp $
+ *  $Id: dirfops.c,v 1.22 2006/02/07 23:21:50 dquigley Exp $
  */
 
 #include "unionfs.h"
@@ -49,7 +48,7 @@ static int unionfs_filldir(void *dirent, const char *name, int namelen,
 	int err = 0;
 	int is_wh_entry = 0;
 
-	dprint(PRINT_DEBUG, "unionfs_filldir name=%*s\n", namelen, name);
+	fist_dprint(9, "unionfs_filldir name=%*s\n", namelen, name);
 
 	buf->filldir_called++;
 
@@ -106,13 +105,13 @@ static int unionfs_readdir(struct file *file, void *dirent, filldir_t filldir)
 
 	print_entry("file = %p, pos = %llx", file, file->f_pos);
 
-	print_file("In unionfs_readdir()", file);
+	fist_print_file("In unionfs_readdir()", file);
 
 	if ((err = unionfs_file_revalidate(file, 0)))
 		goto out;
 
 	inode = file->f_dentry->d_inode;
-	checkinode(inode, "unionfs_readdir");
+	fist_checkinode(inode, "unionfs_readdir");
 
 	uds = ftopd(file)->rdstate;
 	if (!uds) {
@@ -135,7 +134,7 @@ static int unionfs_readdir(struct file *file, void *dirent, filldir_t filldir)
 	while (uds->uds_bindex <= bend) {
 		hidden_file = ftohf_index(file, uds->uds_bindex);
 		if (!hidden_file) {
-			dprint(PRINT_DEBUG,
+			fist_dprint(7,
 				    "Incremented bindex to %d of %d,"
 				    " because hidden file is NULL.\n",
 				    uds->uds_bindex, bend);
@@ -159,10 +158,10 @@ static int unionfs_readdir(struct file *file, void *dirent, filldir_t filldir)
 			err = offset;
 			goto out;
 		}
-		dprint(PRINT_DEBUG, "calling readdir for %d.%lld (offset = %lld)\n",
+		fist_dprint(7, "calling readdir for %d.%lld (offset = %lld)\n",
 			    uds->uds_bindex, uds->uds_dirpos, offset);
 		err = vfs_readdir(hidden_file, unionfs_filldir, (void *)&buf);
-		dprint(PRINT_DEBUG,
+		fist_dprint(7,
 			    "readdir on %d.%lld = %d (entries written %d, filldir called %d)\n",
 			    uds->uds_bindex, (long long)uds->uds_dirpos, err,
 			    buf.entries_written, buf.filldir_called);
@@ -193,7 +192,7 @@ static int unionfs_readdir(struct file *file, void *dirent, filldir_t filldir)
 	}
 
 	if (!buf.filldir_error && uds->uds_bindex >= bend) {
-		dprint(PRINT_DEBUG,
+		fist_dprint(3,
 			    "Discarding rdstate because readdir is over (hashsize = %d)\n",
 			    uds->uds_hashentries);
 		/* Save the number of hash entries for next time. */
@@ -203,12 +202,12 @@ static int unionfs_readdir(struct file *file, void *dirent, filldir_t filldir)
 		file->f_pos = DIREOF;
 	} else {
 		file->f_pos = rdstate2offset(uds);
-		dprint(PRINT_DEBUG, "rdstate now has a cookie of %u (err = %d)\n",
+		fist_dprint(3, "rdstate now has a cookie of %u (err = %d)\n",
 			    uds->uds_cookie, err);
 	}
 
       out:
-	checkinode(inode, "post unionfs_readdir");
+	fist_checkinode(inode, "post unionfs_readdir");
 	print_exit_status(err);
 	return err;
 }
@@ -307,7 +306,9 @@ struct file_operations unionfs_dir_fops = {
 	.llseek = unionfs_dir_llseek,
 	.read = generic_read_dir,
 	.readdir = unionfs_readdir,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
 	.unlocked_ioctl = unionfs_ioctl,
+#endif
 	.open = unionfs_open,
 	.release = unionfs_file_release,
 	.flush = unionfs_flush,
