@@ -1,8 +1,7 @@
 /*
  * Copyright (c) 2003-2006 Erez Zadok
  * Copyright (c) 2003-2006 Charles P. Wright
- * Copyright (c) 2005-2006 Josef 'Jeff' Sipek
- * Copyright (c) 2005-2006 Junjiro Okajima
+ * Copyright (c) 2005-2006 Josef Sipek
  * Copyright (c) 2005      Arun M. Krishnakumar
  * Copyright (c) 2005-2006 David P. Quigley
  * Copyright (c) 2003-2004 Mohammad Nayyer Zubair
@@ -17,7 +16,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: inode.c,v 1.273 2006/08/05 01:28:46 jro Exp $
+ *  $Id: inode.c,v 1.263.2.2 2006/05/30 03:22:06 jsipek Exp $
  */
 
 #include "unionfs.h"
@@ -27,8 +26,12 @@ extern struct dentry *unionfs_lookup(struct inode *, struct dentry *,
 				     struct nameidata *);
 extern int unionfs_readlink(struct dentry *dentry, char __user * buf,
 			    int bufsiz);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,13)
+extern void unionfs_put_link(struct dentry *dentry, struct nameidata *nd);
+#else
 extern void unionfs_put_link(struct dentry *dentry, struct nameidata *nd,
 			     void *cookie);
+#endif
 
 static int unionfs_create(struct inode *parent, struct dentry *dentry,
 			  int mode, struct nameidata *nd)
@@ -43,7 +46,7 @@ static int unionfs_create(struct inode *parent, struct dentry *dentry,
 
 	print_entry_location();
 	lock_dentry(dentry);
-	print_dentry("IN unionfs_create", dentry);
+	fist_print_dentry("IN unionfs_create", dentry);
 
 	/* We start out in the leftmost branch. */
 	bstart = dbstart(dentry);
@@ -78,7 +81,7 @@ static int unionfs_create(struct inode *parent, struct dentry *dentry,
 		struct dentry *hidden_dir_dentry;
 		struct iattr newattrs;
 
-		mutex_lock(&whiteout_dentry->d_inode->i_mutex);
+		down(&whiteout_dentry->d_inode->i_sem);
 		newattrs.ia_valid = ATTR_CTIME | ATTR_MODE | ATTR_ATIME
 		    | ATTR_MTIME | ATTR_UID | ATTR_GID | ATTR_FORCE
 		    | ATTR_KILL_SUID | ATTR_KILL_SGID;
@@ -94,7 +97,7 @@ static int unionfs_create(struct inode *parent, struct dentry *dentry,
 
 		err = notify_change(whiteout_dentry, &newattrs);
 
-		mutex_unlock(&whiteout_dentry->d_inode->i_mutex);
+		up(&whiteout_dentry->d_inode->i_sem);
 
 		if (err)
 			printk(KERN_WARNING
@@ -165,7 +168,7 @@ static int unionfs_create(struct inode *parent, struct dentry *dentry,
 			}
 		}
 
-		checkinode(parent, "unionfs_create");
+		fist_checkinode(parent, "unionfs_create");
 
 		hidden_parent_dentry = lock_parent(hidden_dentry);
 		if (IS_ERR(hidden_parent_dentry)) {
@@ -202,7 +205,7 @@ static int unionfs_create(struct inode *parent, struct dentry *dentry,
 	DPUT(whiteout_dentry);
 	KFREE(name);
 
-	print_dentry("OUT unionfs_create :", dentry);
+	fist_print_dentry("OUT unionfs_create :", dentry);
 	unlock_dentry(dentry);
 	print_exit_status(err);
 	return err;
@@ -354,7 +357,7 @@ static int unionfs_symlink(struct inode *dir, struct dentry *dentry,
 
 	print_entry_location();
 	lock_dentry(dentry);
-	print_dentry("IN unionfs_symlink", dentry);
+	fist_print_dentry("IN unionfs_symlink", dentry);
 
 	/* We start out in the leftmost branch. */
 	bstart = dbstart(dentry);
@@ -383,8 +386,8 @@ static int unionfs_symlink(struct inode *dir, struct dentry *dentry,
 		/* found a .wh.foo entry, unlink it and then call vfs_symlink() */
 		hidden_dir_dentry = lock_parent(whiteout_dentry);
 
-		print_dentry("HDD", hidden_dir_dentry);
-		print_dentry("WD", whiteout_dentry);
+		fist_print_generic_dentry("HDD", hidden_dir_dentry);
+		fist_print_generic_dentry("WD", whiteout_dentry);
 
 		if (!(err = is_robranch_super(dentry->d_sb, bstart))) {
 			err =
@@ -422,7 +425,7 @@ static int unionfs_symlink(struct inode *dir, struct dentry *dentry,
 				if (IS_ERR(hidden_dentry)) {
 					err = PTR_ERR(hidden_dentry);
 				}
-				dprint(PRINT_DEBUG,
+				fist_dprint(8,
 					    "hidden dentry NULL (or error) for bindex = %d\n",
 					    bindex);
 				continue;
@@ -461,7 +464,7 @@ static int unionfs_symlink(struct inode *dir, struct dentry *dentry,
 		d_drop(dentry);
 
 	KFREE(name);
-	print_dentry("OUT unionfs_symlink :", dentry);
+	fist_print_dentry("OUT unionfs_symlink :", dentry);
 	unlock_dentry(dentry);
 	print_exit_status(err);
 	return err;
@@ -480,7 +483,7 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 
 	print_entry_location();
 	lock_dentry(dentry);
-	print_dentry("IN unionfs_mkdir", dentry);
+	fist_print_dentry("IN unionfs_mkdir", dentry);
 	bstart = dbstart(dentry);
 
 	hidden_dentry = dtohd(dentry);
@@ -532,13 +535,13 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 			whiteout_unlinked = 1;
 		}
 	}
-
+	
 	for (bindex = bstart; bindex >= 0; bindex--) {
 		hidden_dentry = dtohd_index(dentry, bindex);
 		if (!hidden_dentry) {
 			hidden_dentry = create_parents(parent, dentry, bindex);
 			if (!hidden_dentry || IS_ERR(hidden_dentry)) {
-				dprint(PRINT_DEBUG,
+				fist_dprint(8,
 					    "hidden dentry NULL for bindex = %d\n",
 					    bindex);
 				continue;
@@ -565,7 +568,7 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 		} else {
 			int i;
 			int bend = dbend(dentry);
-
+			
 			for (i = bindex + 1; i < bend; i++) {
 				if (dtohd_index(dentry, i)) {
 					DPUT(dtohd_index(dentry, i));
@@ -583,10 +586,23 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 				/* update number of links on parent directory */
 				parent->i_nlink = get_nlinks(parent);
 			}
+			whiteout_dentry = LOOKUP_ONE_LEN(UNIONFS_DIR_OPAQUE,
+							 hidden_dentry,
+							 sizeof
+							 (UNIONFS_DIR_OPAQUE) -
+							 1);
+			if (IS_ERR(whiteout_dentry)) {
+				err = PTR_ERR(whiteout_dentry);
+				goto out;
+			}
+			down(&hidden_dentry->d_inode->i_sem);
+			err = vfs_create(hidden_dentry->d_inode,
+					 whiteout_dentry, 0600, NULL);
+			up(&hidden_dentry->d_inode->i_sem);
+			DPUT(whiteout_dentry);
 
-			err = make_dir_opaque(dentry, dbstart(dentry));
 			if (err) {
-				dprint(PRINT_DEBUG,
+				fist_dprint(8,
 					    "mkdir: error creating directory override entry: %d\n",
 					    err);
 				goto out;
@@ -601,7 +617,7 @@ static int unionfs_mkdir(struct inode *parent, struct dentry *dentry, int mode)
 
 	KFREE(name);
 
-	print_dentry("OUT unionfs_mkdir :", dentry);
+	fist_print_dentry("OUT unionfs_mkdir :", dentry);
 	unlock_dentry(dentry);
 	print_exit_status(err);
 	return err;
@@ -619,7 +635,7 @@ static int unionfs_mknod(struct inode *dir, struct dentry *dentry, int mode,
 
 	print_entry_location();
 	lock_dentry(dentry);
-	print_dentry("IN unionfs_mknod", dentry);
+	fist_print_dentry("IN unionfs_mknod", dentry);
 	bstart = dbstart(dentry);
 
 	hidden_dentry = dtohd(dentry);
@@ -669,7 +685,7 @@ static int unionfs_mknod(struct inode *dir, struct dentry *dentry, int mode,
 		if (!hidden_dentry) {
 			hidden_dentry = create_parents(dir, dentry, bindex);
 			if (!hidden_dentry || IS_ERR(hidden_dentry)) {
-				dprint(PRINT_DEBUG,
+				fist_dprint(8,
 					    "hidden dentry NULL for bindex = %d\n",
 					    bindex);
 				continue;
@@ -714,7 +730,7 @@ static int unionfs_mknod(struct inode *dir, struct dentry *dentry, int mode,
 		KFREE(name);
 	}
 
-	print_dentry("OUT unionfs_mknod :", dentry);
+	fist_print_dentry("OUT unionfs_mknod :", dentry);
 	unlock_dentry(dentry);
 	print_exit_status(err);
 	return err;
@@ -728,7 +744,7 @@ int unionfs_readlink(struct dentry *dentry, char __user * buf, int bufsiz)
 	print_entry_location();
 	lock_dentry(dentry);
 	hidden_dentry = dtohd(dentry);
-	print_dentry("unionfs_readlink IN", dentry);
+	fist_print_dentry("unionfs_readlink IN", dentry);
 
 	if (!hidden_dentry->d_inode->i_op ||
 	    !hidden_dentry->d_inode->i_op->readlink) {
@@ -748,7 +764,11 @@ int unionfs_readlink(struct dentry *dentry, char __user * buf, int bufsiz)
 }
 
 /* We don't lock the dentry here, because readlink does the heavy lifting. */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,13)
+static int unionfs_follow_link(struct dentry *dentry, struct nameidata *nd)
+#else
 static void *unionfs_follow_link(struct dentry *dentry, struct nameidata *nd)
+#endif
 {
 	char *buf;
 	int len = PAGE_SIZE, err;
@@ -779,10 +799,18 @@ static void *unionfs_follow_link(struct dentry *dentry, struct nameidata *nd)
 
       out:
 	print_exit_status(err);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,13)
+	return err;
+#else
 	return ERR_PTR(err);
+#endif
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,13)
+void unionfs_put_link(struct dentry *dentry, struct nameidata *nd)
+#else
 void unionfs_put_link(struct dentry *dentry, struct nameidata *nd, void *cookie)
+#endif
 {
 	char *link;
 	print_entry_location();
@@ -826,12 +854,20 @@ static int inode_permission(struct inode *inode, int mask, struct nameidata *nd,
 		    (!strcmp("nfs", (inode)->i_sb->s_type->name)) &&
 		    (nd) && (nd->mnt) && (nd->mnt->mnt_sb) &&
 		    (branchperms(nd->mnt->mnt_sb, bindex) & MAY_NFSRO)) {
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
+			retval = vfs_permission(inode, submask);
+#else
 			retval = generic_permission(inode, submask, NULL);
+#endif
 		}
 	} else {
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,10)
+		retval = vfs_permission(inode, submask);
+#else
 		retval = generic_permission(inode, submask, NULL);
+#endif
 	}
-
+	
 	if (retval && retval != -EROFS) /* ignore EROFS */
 		return retval;
 
@@ -853,7 +889,7 @@ static int unionfs_permission(struct inode *inode, int mask,
 	bstart = ibstart(inode);
 	bend = ibend(inode);
 
-	print_inode("IN unionfs_permission", inode);
+	fist_print_inode("IN unionfs_permission", inode);
 
 	for (bindex = bstart; bindex <= bend; bindex++) {
 		hidden_inode = itohi_index(inode, bindex);
@@ -866,7 +902,7 @@ static int unionfs_permission(struct inode *inode, int mask,
 		 */
 		if (!is_file && !S_ISDIR(hidden_inode->i_mode))
 			continue;
-		/* We use our own special version of permission, such that
+		/* We use our own special version of permission, such that 
 		 * only the first branch returns -EROFS. */
 		err = inode_permission(hidden_inode, mask, nd, bindex);
 		/* The permissions are an intersection of the overall directory
@@ -941,30 +977,15 @@ static int unionfs_setattr(struct dentry *dentry, struct iattr *ia)
 			goto out;
 		break;
 	}
-#ifdef UNIONFS_MMAP
-	/*
-	 * SP: notify_change will change the lower file's size,
-	 * but we need to truncate the page tables, so need to call
-	 * vmtruncate()
-	 */
 
-	if (ia->ia_valid & ATTR_SIZE) {
-		if (ia->ia_size != i_size_read(inode)) {
-			err = vmtruncate(inode, ia->ia_size);
-			if (err) {
-				printk("unionfs_setattr: vmtruncate failed\n");
-			}
-		}
-	}
-#endif
 	/* get the size from the first hidden inode */
 	hidden_inode = itohi(dentry->d_inode);
-	checkinode(inode, "unionfs_setattr");
+	fist_checkinode(inode, "unionfs_setattr");
 	fist_copy_attr_all(inode, hidden_inode);
 
       out:
 	unlock_dentry(dentry);
-	checkinode(inode, "post unionfs_setattr");
+	fist_checkinode(inode, "post unionfs_setattr");
 	print_exit_status(err);
 	return err;
 }
