@@ -16,7 +16,7 @@
  * This Copyright notice must be kept intact and distributed with all sources.
  */
 /*
- *  $Id: main.c,v 1.161 2006/02/20 22:23:01 dquigley Exp $
+ *  $Id: main.c,v 1.161.2.2 2006/04/23 19:55:52 jsipek Exp $
  */
 
 #include "unionfs.h"
@@ -387,8 +387,19 @@ static int parse_dirs_option(struct super_block *sb, struct unionfs_dentry_info
 
       out:
 	if (err) {
+		for (i = 0; i < branches; i++) {
+			if (hidden_root_info->udi_dentry[i])
+				DPUT(hidden_root_info->udi_dentry[i]);
+		}
+
 		KFREE(hidden_root_info->udi_dentry);
 		KFREE(stopd(sb)->usi_data);
+
+		/* MUST clear the pointers to prevent potential double free if
+		 * the caller dies later on
+		 */
+		hidden_root_info->udi_dentry = NULL;
+		stopd(sb)->usi_data = NULL;
 	}
 	return err;
 }
@@ -531,14 +542,18 @@ static struct unionfs_dentry_info *unionfs_parse_options(struct super_block *sb,
 	goto out;
 
       out_error:
-	for (bindex = hidden_root_info->udi_bstart;
-	     bindex >= 0 && bindex <= hidden_root_info->udi_bend; bindex++) {
-		struct dentry *d;
-		d = hidden_root_info->udi_dentry[bindex];
-		DPUT(d);
-		if (stohiddenmnt_index(sb, bindex))
-			mntput(stohiddenmnt_index(sb, bindex));
+	if (hidden_root_info && hidden_root_info->udi_dentry) {
+		for (bindex = hidden_root_info->udi_bstart;
+		     bindex >= 0 && bindex <= hidden_root_info->udi_bend;
+		     bindex++) {
+			struct dentry *d;
+			d = hidden_root_info->udi_dentry[bindex];
+			DPUT(d);
+			if (stohiddenmnt_index(sb, bindex))
+				mntput(stohiddenmnt_index(sb, bindex));
+		}
 	}
+
 	KFREE(hidden_root_info->udi_dentry);
 	KFREE(hidden_root_info);
 
