@@ -1,12 +1,20 @@
 #!/bin/bash
-#===================================================#
-# Make One Linux -- mkkmodule script                #
-#  (C)2005 Keicho Kondo <dgel@users.sourceforge.jp> #
-#===================================================#
-# This script generates kernel module.
 #
-# This file may be copied under the terms of the GNU Public License
-# http://www.gnu.org/licenses/gpl.html
+# MKKMODULE -- This script generates a kernel module.
+# Copyright (C) 2005-2006 Keicho Kondo <dgel@users.sourceforge.jp>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 # To save time the script will look for a relevant Module.symvers in:
 #   /lib/modules
@@ -156,22 +164,23 @@ KERNEL_SRCDIR=""
 TYPE=""
 if [ -f "${RPM_TOPDIR}/SOURCES/linux-"*".tar."*"" -a -f "${RPM_TOPDIR}/SPECS/kernel-"*".spec" ]
 then
-	echo "	${RPM_TOPDIR}"
+	#echo "	${RPM_TOPDIR}"
 	rpmbuild -bp --target=noarch ${RPM_TOPDIR}/SPECS/kernel-*.spec >/dev/null 2>&1
 	if [ $? -ne 0 ]
 	then
 		echo " failed.  Giving up."
 		exit 1
 	fi
-	KERNEL_SRCDIR="$RPM_TOPDIR/BUILD/kernel-*/linux-*"
+	KERNEL_SRCDIR="$(echo $RPM_TOPDIR/BUILD/kernel-*/linux-*)"
 elif [ -d "/usr/src/linux" ]
 then
-	echo "	/usr/src/linux"
+	#echo "	/usr/src/linux"
 	KERNEL_SRCDIR="/usr/src/linux"
 else
-	echo ""
+	#echo ""
 	KERNEL_SRCDIR="(not_found)"
 fi
+echo -e "\t${KERNEL_SRCDIR}"
 if ! chkkernelsource ${KERNEL_SRCDIR}
 then
 	getkernelsource
@@ -207,7 +216,7 @@ then
 		[ -z "$f" ] && continue
 		case "${RELEASE}" in
 		*$f)
-			SUFF="$f" ;;
+			SUFF="-$f" ;;
 		esac
 	done
 	CPU=$(uname -p)
